/*
 * Copyright 1999-2017 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.polardbx.druid.bvt.sql.mysql;

import com.alibaba.polardbx.druid.sql.SQLUtils;
import com.alibaba.polardbx.druid.sql.ast.SQLStatement;
import com.alibaba.polardbx.druid.sql.dialect.mysql.parser.MySqlStatementParser;
import com.alibaba.polardbx.druid.sql.parser.SQLStatementParser;
import com.alibaba.polardbx.druid.util.JdbcConstants;
import junit.framework.TestCase;
import org.junit.Assert;

import java.util.List;

public class START_TRANSACTION_Test extends TestCase {

    public void test_0() throws Exception {
        String sql = "START TRANSACTION;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION;", text);
    }

    public void test_1() throws Exception {
        String sql = "START TRANSACTION WITH CONSISTENT SNAPSHOT;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION WITH CONSISTENT SNAPSHOT;", text);
    }

    public void test_2() throws Exception {
        String sql = "START TRANSACTION BEGIN;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION BEGIN;", text);
    }

    public void test_3() throws Exception {
        String sql = "START TRANSACTION BEGIN WORK;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION BEGIN WORK;", text);
    }

    public void test_4() throws Exception {
        String sql = "COMMIT;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("COMMIT;", text);
    }

    public void test_5() throws Exception {
        String sql = "COMMIT WORK;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("COMMIT WORK;", text);
    }

    public void test_6() throws Exception {
        String sql = "ROLLBACK;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("ROLLBACK;", text);
    }

    public void test_7() throws Exception {
        String sql = "SET autocommit=0;";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("SET autocommit = 0;", text);
    }

    public void test_8() throws Exception {
        String sql = "START TRANSACTION ISOLATION LEVEL READ UNCOMMITTED";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION ISOLATION LEVEL READ UNCOMMITTED", text);
    }

    public void test_9() throws Exception {
        String sql = "START TRANSACTION ISOLATION LEVEL READ COMMITTED";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION ISOLATION LEVEL READ COMMITTED", text);
    }

    public void test_10() throws Exception {
        String sql = "START TRANSACTION ISOLATION LEVEL REPEATABLE READ";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION ISOLATION LEVEL REPEATABLE READ", text);
    }

    public void test_11() throws Exception {
        String sql = "START TRANSACTION ISOLATION LEVEL SERIALIZABLE";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION ISOLATION LEVEL SERIALIZABLE", text);
    }

    public void test_12() throws Exception {
        String sql = "start transaction read only";

        SQLStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();

        String text = output(stmtList);

        Assert.assertEquals("START TRANSACTION READ ONLY", text);
    }

    private String output(List<SQLStatement> stmtList) {
        return SQLUtils.toSQLString(stmtList, JdbcConstants.MYSQL);
    }
}
